package geniusweb.protocol.session;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

import geniusweb.references.PartyWithParameters;
import geniusweb.references.PartyWithProfile;
import geniusweb.references.ProfileRef;

/**
 * Contains a functional team, eg in SHAOP a team would be a SHAOP together with
 * his COB partner. In SAOP this is just a SAOP party.
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE)
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.WRAPPER_OBJECT)
public class TeamInfo {

	private List<PartyWithProfile> parties;

	/**
	 * Convenient constructor if team is only 1 party and profile
	 * 
	 * @param party   the {@link PartyWithParameters}
	 * @param profile the ProfileRef to be used by party
	 */
	public TeamInfo(PartyWithParameters party, ProfileRef profile) {
		this(Arrays.asList(new PartyWithProfile(party, profile)));
	}

	@JsonCreator
	public TeamInfo(@JsonProperty("teams") List<PartyWithProfile> teams) {
		this.parties = teams;
	}

	/**
	 * 
	 * @return the list of all parties involved, which is the parties in all
	 *         teams combined.
	 */
	public List<PartyWithProfile> getAllParties() {
		return Collections.unmodifiableList(parties);
	}

	/**
	 * 
	 * @return the number of parties in the team.
	 */
	public Integer getSize() {
		return parties.size();
	}

	@Override
	public String toString() {
		return "TeamInfo[" + parties + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((parties == null) ? 0 : parties.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		TeamInfo other = (TeamInfo) obj;
		if (parties == null) {
			if (other.parties != null)
				return false;
		} else if (!parties.equals(other.parties))
			return false;
		return true;
	}
}
